# Copyright 2016: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import jinja2
import six

from muranopkgcheck import error

TEMPLATE = """
============  ================================================================
    CODE                           DESCRITION
============  ================================================================
{% for err in errors %}{{ err.code }}          {{err.description}}
{% endfor %}
============  ================================================================
"""


def main():
    from muranopkgcheck import validators
    print('Validators:')
    for v in validators.VALIDATORS:
        print(v.__name__)
    errors = sorted(six.itervalues(error.errors),
                    key=lambda item: item['code'])
    t = jinja2.Template(TEMPLATE)
    r = t.render(errors=errors)
    print(r)
    with open('doc/source/_errors_list.rst', 'w') as f:
        f.write(r)

if __name__ == '__main__':
    main()
